;    rgbwall.asm: firmware for a 6-channel serial-controlled PWM-controller
;    Copyright 2010 J. Domburg
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.


.device ATTiny2313
.include "tiny2313.inc"
;FUSE_L=0xFF
;FUSE_H=0xDB

;Port B
.equ led1	=	0
.equ led2	=	1
.equ led3	=	2


;Port D
.equ led4	=	5
.equ led5	=	4
.equ led6	=	3


.def led1val	=	r1
.def led2val	=	r2
.def led3val	=	r3
.def led4val	=	r4
.def led5val	=	r5
.def led6val	=	r6
.def temp		=	R16 	;The temp register
.def temp2		=	R17 	;Second temp register
.def pwmctr		=	r18

.def led1sval	=	r19
.def led2sval	=	r20
.def led3sval	=	r21
.def led4sval	=	r22
.def led5sval	=	r23
.def led6sval	=	r24
.def changevals	=	r25

	rjmp start		; reset vector
	reti			; External Int 0 not enabled
	reti			; int1
	reti
	reti
	reti
	rjmp tim0int	; Timer 0 Overflow Int not enabled
	reti			; Eeprom ready not enabled
	reti			; Anacomp interrupt Not enabled
	reti			; Timer 0 comparator A not enabled
	reti			; Timer 0 comparator B not enabled
	reti			; ADC conversion complete Not enabled
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

start:
	ldi temp,RAMEND		; top of memory
	out SPL,temp		; init stack pointer
	ldi temp,0b00000111	; Port B direction options
	out DDRB,temp		; Setup port direction
	ldi temp,0b11100000	;
	out PORTB,temp		; Enable pull-ups on the address lines
	ldi temp,0b00111000	; Port B direction options
	out DDRD,temp		; Setup port direction
	;Init timer: no divider, int at overflow
	ldi temp,0
	out tccr0a,temp
	ldi temp,1
	out TCCR0B,temp
	ldi temp,2
	out TIMSK,temp
	;Init uart: recv @ 19200,n,8,1 , no ints
	ldi temp,0
	out UCSRA,temp
	ldi temp,$10
	out UCSRB,temp
	ldi temp,$6
	out UCSRC,temp
	ldi temp,0
	out UBRRH,temp
	ldi temp,64
	out UBRRL,temp

	;Default to all on. Handy for after power failures.
	ldi led1sval,0
	ldi led2sval,0
	ldi led3sval,0
	ldi led4sval,0
	ldi led5sval,0
	ldi led6sval,0
	ldi changevals,1

;Main loop. Handles communication.
loop:
	sei

;Wait for start byte
	rcall w4recv
	cpi temp,$aa
	brne loop
	rcall w4recv
	cpi temp,$55
	brne loop

;Ok, start byte is received.
;Wait (addr*6) bytes to skip the data for the pcb before us
	rcall getaddr
	mov temp2,temp
skiploop1:
	cpi temp2,0
	breq noskip1
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	dec temp2
	rjmp skiploop1
noskip1:


;Get 6 RGB-bytes and invert them: For the PWM-routines, 255 = completely off, 0=completely on.
	ldi led1sval,255
	ldi led2sval,255
	ldi led3sval,255
	ldi led4sval,255
	ldi led5sval,255
	ldi led6sval,255
	rcall w4recv
	sub led1sval, temp
	rcall w4recv
	sub led2sval, temp
	rcall w4recv
	sub led3sval, temp
	rcall w4recv
	sub led4sval, temp
	rcall w4recv
	sub led5sval, temp
	rcall w4recv
	sub led6sval, temp


;Wait another (7-addr)*6 bytes (for the next pcbs)
	rcall getaddr
	ldi temp2,7
	sub temp2,temp
skiploop2:
	cpi temp2,0
	breq noskip2
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	rcall w4recv
	dec temp2
	rjmp skiploop2
noskip2:

;Ok, all done.
	ldi changevals,1
	rjmp loop

;Subroutine:	
;Get the address of this module (=PB7-5)
getaddr:
	in temp, pinb
	andi temp,0b11100000
	swap temp
	lsr temp
	ret


;Subroutine:
;Waits till the UART has received a byte and returns it in temp.
w4recv:
	sbis UCSRA,7
	 rjmp w4recv
	in temp,UDR
	ret


;Timer 0 int. Do software pwm as quickly as possible
tim0int:
	push temp
	in temp,sreg
	push temp
	
;Schedule another timer int in 50 cycles. This should in the worst case scenario
;leave the main program running for 20 cycles after the int is over.
	ldi temp,255-50
	out tcnt0,temp
	
	inc pwmctr
	cpi pwmctr,255
	breq pwmzero

	cp pwmctr,led1val
	brne noled1
	sbi portb,led1
noled1:
	cp pwmctr,led2val
	brne noled2
	sbi portb,led2
noled2:
	cp pwmctr,led3val
	brne noled3
	sbi portb,led3
noled3:
	cp pwmctr,led4val
	brne noled4
	sbi portd,led4
noled4:
	cp pwmctr,led5val
	brne noled5
	sbi portd,led5
noled5:
	cp pwmctr,led6val
	brne noled6
	sbi portd,led6
noled6:

	pop temp
	out sreg,temp
	pop temp
	reti

pwmzero:
	;Pwm-val is reset. Handle that.
	cpi changevals,0
	breq nochangevals
	;Apply changes
	mov led1val,led1sval
	mov led2val,led2sval
	mov led3val,led3sval
	mov led4val,led4sval
	mov led5val,led5sval
	mov led6val,led6sval
	ldi changevals,0
nochangevals:

	cbi portb,led1
	cbi portb,led2
	cbi portb,led3
	cbi portd,led4
	cbi portd,led5
	cbi portd,led6

	pop temp
	out sreg,temp
	pop temp
	reti
	

