//Compile with
//gcc -lm -o rgbwall main.c

/*
	Copyright 2010 J. Domburg
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <sys/time.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <math.h>

#define BAUDRATE B19200
#define _POSIX_SOURCE 1 /* POSIX compliant source */
       
int com;

typedef struct {
	unsigned char r;
	unsigned char g;
	unsigned char b;
} rgbType;

void openCom() {
    struct timeval tv;
    struct termios oldtio,newtio;

    com = open("/dev/ttyUSB0", O_RDWR | O_NOCTTY );
    if (com <0) {perror("Opening comport"); exit(-1); }
    tcgetattr(com,&oldtio); // save current port settings
    bzero(&newtio, sizeof(newtio));
    newtio.c_cflag =  (CS8 | CREAD | CLOCAL);
    newtio.c_lflag = 0;
    newtio.c_iflag = IGNPAR;
    newtio.c_oflag = 0;
    cfsetospeed(&newtio,BAUDRATE);
    
    newtio.c_cc[VTIME]    = 0;   // inter-character timer unused 
    newtio.c_cc[VMIN]     = 1;   // blocking read until 1 chars received 
      
    tcflush(com,TCIFLUSH);
    tcsetattr(com,TCSANOW,&newtio);
}

void sendRgb(rgbType val[]) {
	//Address translation: use this if the boards aren't lined up incrementally
	int addr[16]={4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 2, 3, 0, 1};
	unsigned char buff[6*8+2];
	int x;
	int ret;
	buff[0]=0xaa;
	buff[1]=0x55;
	for (x=0; x<16; x++) {
		buff[addr[x]*3+2]=val[x].r;
		buff[addr[x]*3+3]=val[x].g;
		buff[addr[x]*3+4]=val[x].b;
	}
	ret=write(com, buff, 6*8+2);
	if (ret!=6*8+2) {
		perror("writing to serial port");
		exit(0);
	}
}

unsigned char interpolate(unsigned char col1, unsigned char col2, float p) {
	float r=((float)col1*p)+((float)col2*(1-p));
	return r;
}


int main(int argc, char **argv) {
	rgbType col[16];
	openCom();
	
	if (strcmp(argv[1], "hue")==0) {
		double p=0;
		int x;
		while(1) {
			float phase=.1;
			for (x=0; x<16; x++) {
				col[x].r=(sin(p+(x*phase)+0)+1)*127;
				col[x].g=(sin(p+(x*phase)+2.1)+1)*127;
				col[x].b=(sin(p+(x*phase)+4.2)+1)*127;
			}
			sendRgb(col);
			p+=.002;
		}
	} else if (strcmp(argv[1], "grad")==0) {
		long col1, col2;
		int x;
		col1=strtol(argv[2], NULL, 16);
		if (argc==4) {
			col2=strtol(argv[3], NULL, 16);
		} else {
			col2=col1;
		}
		printf("%06lX -> %06lX\n", col1, col2);
		for (x=0; x<16; x++) {
			col[x].b=interpolate((col1)&0xff, (col2)&0xff, ((float)x/14));
			col[x].g=interpolate((col1>>8)&0xff, (col2>>8)&0xff, ((float)x/14));
			col[x].r=interpolate((col1>>16)&0xff, (col2>>16)&0xff, ((float)x/14));
			printf("%06lX\n",((long)col[x].r<<16)+(col[x].g<<8)+(col[x].b));
		}
		sendRgb(col);
	} else if (strcmp(argv[1], "nr")==0) {
		int x, p, dir, t;
		dir=1; t=0; p=0;
		for (x=0; x<16; x++) {
		    col[x].r=0; col[x].g=0; col[x].b=0;
		}
		while(1) {
			for (x=0; x<16; x++) {
				if (col[x].r>1) col[x].r/=1.3; else col[x].r=0;
			}
			t++; 
			if (t>1) {
				t=0;
				p+=dir;
				if (p==0) dir=1;
				if (p==13) dir=-1;
			}
			col[p].r=255;
			col[p].g=0;
			col[p].b=0;
			sendRgb(col);
		}
	}
	close(com);
}


